@@ -16,21 +16,23 @@ fn main() {
     xplr [FLAG]... [OPTION]... [PATH] [SELECTION]..."###;
 
         let flags = r###"
-    -                  Reads paths from stdin
-    --                 Denotes the end of command-line flags and options
-        --read-only    Enables read-only mode (config.general.read_only)
-    -h, --help         Prints help information
-    -V, --version      Prints version information"###;
+    -                            Reads new-line (\n) separated paths from stdin
+    --                           Denotes the end of command-line flags and options
+        --force-focus            Focuses on the given <PATH>, even if it is a directory
+    -h, --help                   Prints help information
+        --print-pwd-as-result    Prints the present working directory when quitting 
+                                   with `PrintResultAndQuit`
+        --read-only              Enables read-only mode (config.general.read_only)
+        --read0                  Reads paths separated using the null character (\0)
+        --write0                 Prints paths separated using the null character (\0)
+    -0  --null                   Combines --read0 and --write0
+    -V, --version                Prints version information"###;
 
         let options = r###"
     -c, --config <PATH>             Specifies a custom config file (default is
                                       "$HOME/.config/xplr/init.lua")
     -C, --extra-config <PATH>...    Specifies extra config files to load
-        --on-load <MESSAGE>...      Sends messages when xplr loads
-        --force-focus               Focuses on the given <PATH>, even if it is
-                                      a directory
-        --print-pwd-as-result       Prints the last working directory when
-                                      quitting with `PrintResultAndQuit`"###;
+        --on-load <MESSAGE>...      Sends messages when xplr loads"###;
 
         let args = r###"
     <PATH>            Path to focus on, or enter if directory, (default is `.`)
@@ -52,8 +54,18 @@ fn main() {
     } else if cli.version {
         println!("xplr {}", xplr::app::VERSION);
     } else {
+        let write0 = cli.write0;
         match runner::from_cli(cli).and_then(|a| a.run()) {
-            Ok(Some(out)) => print!("{}", out),
+            Ok(Some(mut out)) => {
+                if write0 {
+                    out = out
+                        .trim_end()
+                        .chars()
+                        .map(|c| if c == '\n' { '\0' } else { c })
+                        .collect();
+                }
+                print!("{}", out);
+            }
             Ok(None) => {}
             Err(err) => {
                 if !err.to_string().is_empty() {
