@@ -275,6 +275,9 @@ pub struct GeneralConfig {
 
     #[serde(default)]
     pub start_fifo: Option<String>,
+
+    #[serde(default)]
+    pub global_key_bindings: KeyBindings,
 }
 
 #[derive(Debug, Clone, Default, PartialEq, Serialize, Deserialize)]
@@ -323,25 +326,34 @@ impl KeyBindings {
                 .filter_map(|(k, a)| a.sanitized(read_only).map(|a| (k, a)))
                 .collect();
 
-            self.on_alphabet =
-                self.on_alphabet.and_then(|a| a.sanitized(read_only));
-            self.on_number =
-                self.on_number.and_then(|a| a.sanitized(read_only));
+            self.on_alphabet = self.on_alphabet.and_then(|a| a.sanitized(read_only));
+            self.on_number = self.on_number.and_then(|a| a.sanitized(read_only));
             self.on_alphanumeric =
                 self.on_alphanumeric.and_then(|a| a.sanitized(read_only));
             self.on_special_character = self
                 .on_special_character
                 .and_then(|a| a.sanitized(read_only));
-            self.on_character =
-                self.on_character.and_then(|a| a.sanitized(read_only));
-            self.on_navigation =
-                self.on_navigation.and_then(|a| a.sanitized(read_only));
-            self.on_function =
-                self.on_function.and_then(|a| a.sanitized(read_only));
+            self.on_character = self.on_character.and_then(|a| a.sanitized(read_only));
+            self.on_navigation = self.on_navigation.and_then(|a| a.sanitized(read_only));
+            self.on_function = self.on_function.and_then(|a| a.sanitized(read_only));
             self.default = self.default.and_then(|a| a.sanitized(read_only));
         };
         self
     }
+
+    pub fn extend(mut self, other: Self) -> Self {
+        self.on_key.extend(other.on_key);
+        self.on_alphabet = other.on_alphabet.or(self.on_alphabet);
+        self.on_number = other.on_number.or(self.on_number);
+        self.on_alphanumeric = other.on_alphanumeric.or(self.on_alphanumeric);
+        self.on_special_character =
+            other.on_special_character.or(self.on_special_character);
+        self.on_character = other.on_character.or(self.on_character);
+        self.on_navigation = other.on_navigation.or(self.on_navigation);
+        self.on_function = other.on_function.or(self.on_function);
+        self.default = other.default.or(self.default);
+        self
+    }
 }
 
 #[derive(Debug, Clone, Default, PartialEq, Serialize, Deserialize)]
@@ -364,8 +376,14 @@ pub struct Mode {
 }
 
 impl Mode {
-    pub fn sanitized(mut self, read_only: bool) -> Self {
-        self.key_bindings = self.key_bindings.sanitized(read_only);
+    pub fn sanitized(
+        mut self,
+        read_only: bool,
+        global_key_bindings: KeyBindings,
+    ) -> Self {
+        self.key_bindings = global_key_bindings
+            .sanitized(read_only)
+            .extend(self.key_bindings.sanitized(read_only));
         self
     }
 
@@ -387,36 +405,32 @@ impl Mode {
                 let lines = extra_help_lines
                     .unwrap_or_default()
                     .into_iter()
-                    .chain(self.key_bindings.on_key.iter().filter_map(
-                        |(k, a)| {
-                            let remaps = self
-                                .key_bindings
-                                .on_key
-                                .iter()
-                                .filter_map(|(rk, ra)| {
-                                    if rk == k {
-                                        None
-                                    } else if a == ra {
-                                        Some(rk.clone())
-                                    } else {
-                                        None
-                                    }
-                                })
-                                .collect::<Vec<String>>();
-                            a.help.clone().map(|h| {
-                                HelpMenuLine::KeyMap(k.into(), remaps, h)
+                    .chain(self.key_bindings.on_key.iter().filter_map(|(k, a)| {
+                        let remaps = self
+                            .key_bindings
+                            .on_key
+                            .iter()
+                            .filter_map(|(rk, ra)| {
+                                if rk == k {
+                                    None
+                                } else if a == ra {
+                                    Some(rk.clone())
+                                } else {
+                                    None
+                                }
                             })
-                        },
-                    ))
+                            .collect::<Vec<String>>();
+                        a.help
+                            .clone()
+                            .map(|h| HelpMenuLine::KeyMap(k.into(), remaps, h))
+                    }))
                     .chain(
                         self.key_bindings
                             .on_alphabet
                             .iter()
                             .map(|a| ("[a-Z]", a.help.clone()))
                             .filter_map(|(k, mh)| {
-                                mh.map(|h| {
-                                    HelpMenuLine::KeyMap(k.into(), vec![], h)
-                                })
+                                mh.map(|h| HelpMenuLine::KeyMap(k.into(), vec![], h))
                             }),
                     )
                     .chain(
@@ -425,9 +439,7 @@ impl Mode {
                             .iter()
                             .map(|a| ("[0-9]", a.help.clone()))
                             .filter_map(|(k, mh)| {
-                                mh.map(|h| {
-                                    HelpMenuLine::KeyMap(k.into(), vec![], h)
-                                })
+                                mh.map(|h| HelpMenuLine::KeyMap(k.into(), vec![], h))
                             }),
                     )
                     .chain(
@@ -436,9 +448,7 @@ impl Mode {
                             .iter()
                             .map(|a| ("[0-Z]", a.help.clone()))
                             .filter_map(|(k, mh)| {
-                                mh.map(|h| {
-                                    HelpMenuLine::KeyMap(k.into(), vec![], h)
-                                })
+                                mh.map(|h| HelpMenuLine::KeyMap(k.into(), vec![], h))
                             }),
                     )
                     .chain(
@@ -447,9 +457,7 @@ impl Mode {
                             .iter()
                             .map(|a| ("[^0-Z]", a.help.clone()))
                             .filter_map(|(k, mh)| {
-                                mh.map(|h| {
-                                    HelpMenuLine::KeyMap(k.into(), vec![], h)
-                                })
+                                mh.map(|h| HelpMenuLine::KeyMap(k.into(), vec![], h))
                             }),
                     )
                     .chain(
@@ -458,9 +466,7 @@ impl Mode {
                             .iter()
                             .map(|a| ("[*]", a.help.clone()))
                             .filter_map(|(k, mh)| {
-                                mh.map(|h| {
-                                    HelpMenuLine::KeyMap(k.into(), vec![], h)
-                                })
+                                mh.map(|h| HelpMenuLine::KeyMap(k.into(), vec![], h))
                             }),
                     )
                     .chain(
@@ -469,9 +475,7 @@ impl Mode {
                             .iter()
                             .map(|a| ("[nav]", a.help.clone()))
                             .filter_map(|(k, mh)| {
-                                mh.map(|h| {
-                                    HelpMenuLine::KeyMap(k.into(), vec![], h)
-                                })
+                                mh.map(|h| HelpMenuLine::KeyMap(k.into(), vec![], h))
                             }),
                     )
                     .chain(
@@ -480,9 +484,7 @@ impl Mode {
                             .iter()
                             .map(|a| ("[f1-f12]", a.help.clone()))
                             .filter_map(|(k, mh)| {
-                                mh.map(|h| {
-                                    HelpMenuLine::KeyMap(k.into(), vec![], h)
-                                })
+                                mh.map(|h| HelpMenuLine::KeyMap(k.into(), vec![], h))
                             }),
                     )
                     .chain(
@@ -491,9 +493,7 @@ impl Mode {
                             .iter()
                             .map(|a| ("[default]", a.help.clone()))
                             .filter_map(|(k, mh)| {
-                                mh.map(|h| {
-                                    HelpMenuLine::KeyMap(k.into(), vec![], h)
-                                })
+                                mh.map(|h| HelpMenuLine::KeyMap(k.into(), vec![], h))
                             }),
                     );
 
@@ -521,119 +521,11 @@ impl Mode {
     }
 }
 
-#[derive(Debug, Clone, Default, Serialize, Deserialize)]
-#[serde(deny_unknown_fields)]
-pub struct BuiltinModesConfig {
-    #[serde(default)]
-    pub default: Mode,
-
-    #[serde(default)]
-    pub debug_error: Mode,
-
-    #[serde(default)]
-    pub recover: Mode,
-
-    #[serde(default)]
-    pub selection_ops: Mode,
-
-    #[serde(default)]
-    pub create: Mode,
-
-    #[serde(default)]
-    pub create_directory: Mode,
-
-    #[serde(default)]
-    pub create_file: Mode,
-
-    #[serde(default)]
-    pub number: Mode,
-
-    #[serde(default)]
-    pub go_to: Mode,
-
-    #[serde(default)]
-    pub rename: Mode,
-
-    #[serde(default)]
-    pub duplicate_as: Mode,
-
-    #[serde(default)]
-    pub delete: Mode,
-
-    #[serde(default)]
-    pub action: Mode,
-
-    #[serde(default)]
-    pub search: Mode,
-
-    #[serde(default)]
-    pub filter: Mode,
-
-    #[serde(default)]
-    pub relative_path_does_match_regex: Mode,
-
-    #[serde(default)]
-    pub relative_path_does_not_match_regex: Mode,
-
-    #[serde(default)]
-    pub sort: Mode,
-
-    #[serde(default)]
-    pub switch_layout: Mode,
-
-    #[serde(default)]
-    pub quit: Mode,
-}
-
-impl BuiltinModesConfig {
-    pub fn get(&self, name: &str) -> Option<&Mode> {
-        match name {
-            "default" => Some(&self.default),
-            "debug_error" => Some(&self.debug_error),
-            "recover" => Some(&self.recover),
-            "selection ops" => Some(&self.selection_ops),
-            "selection_ops" => Some(&self.selection_ops),
-            "create" => Some(&self.create),
-            "create file" => Some(&self.create_file),
-            "create_file" => Some(&self.create_file),
-            "create directory" => Some(&self.create_directory),
-            "create_directory" => Some(&self.create_directory),
-            "number" => Some(&self.number),
-            "go to" => Some(&self.go_to),
-            "go_to" => Some(&self.go_to),
-            "rename" => Some(&self.rename),
-            "duplicate as" => Some(&self.duplicate_as),
-            "duplicate_as" => Some(&self.duplicate_as),
-            "delete" => Some(&self.delete),
-            "action" => Some(&self.action),
-            "search" => Some(&self.search),
-            "sort" => Some(&self.sort),
-            "filter" => Some(&self.filter),
-            "relative_path_does_match_regex" => {
-                Some(&self.relative_path_does_match_regex)
-            }
-            "relative path does match regex" => {
-                Some(&self.relative_path_does_match_regex)
-            }
-            "relative_path_does_not_match_regex" => {
-                Some(&self.relative_path_does_not_match_regex)
-            }
-            "relative path does not match regex" => {
-                Some(&self.relative_path_does_not_match_regex)
-            }
-            "switch layout" => Some(&self.switch_layout),
-            "switch_layout" => Some(&self.switch_layout),
-            "quit" => Some(&self.quit),
-            _ => None,
-        }
-    }
-}
-
 #[derive(Debug, Clone, Default, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct ModesConfig {
     #[serde(default)]
-    pub builtin: BuiltinModesConfig,
+    pub builtin: HashMap<String, Mode>,
 
     #[serde(default)]
     pub custom: HashMap<String, Mode>,
@@ -675,42 +567,11 @@ impl PanelUiConfig {
     }
 }
 
-#[derive(Debug, Clone, Default, Serialize, Deserialize)]
-#[serde(deny_unknown_fields)]
-pub struct BuiltinLayoutsConfig {
-    #[serde(default)]
-    pub default: Layout,
-
-    #[serde(default)]
-    pub no_help: Layout,
-
-    #[serde(default)]
-    pub no_selection: Layout,
-
-    #[serde(default)]
-    pub no_help_no_selection: Layout,
-}
-
-impl BuiltinLayoutsConfig {
-    pub fn get(&self, name: &str) -> Option<&Layout> {
-        match name {
-            "default" => Some(&self.default),
-            "no_help" => Some(&self.no_help),
-            "no help" => Some(&self.no_help),
-            "no_selection" => Some(&self.no_selection),
-            "no selection" => Some(&self.no_selection),
-            "no_help_no_selection" => Some(&self.no_help_no_selection),
-            "no help no selection" => Some(&self.no_help_no_selection),
-            _ => None,
-        }
-    }
-}
-
 #[derive(Debug, Clone, Default, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct LayoutsConfig {
     #[serde(default)]
-    pub builtin: BuiltinLayoutsConfig,
+    pub builtin: HashMap<String, Layout>,
 
     #[serde(default)]
     pub custom: HashMap<String, Layout>,
