@@ -1085,18 +1085,16 @@ impl NodeSorterApplicable {
                     .unwrap_or_default(),
             ),
 
-            NodeSorter::ByICanonicalAbsolutePath => {
-                natord::compare_ignore_case(
-                    &a.canonical
-                        .as_ref()
-                        .map(|s| s.absolute_path.clone())
-                        .unwrap_or_default(),
-                    &b.canonical
-                        .as_ref()
-                        .map(|s| s.absolute_path.clone())
-                        .unwrap_or_default(),
-                )
-            }
+            NodeSorter::ByICanonicalAbsolutePath => natord::compare_ignore_case(
+                &a.canonical
+                    .as_ref()
+                    .map(|s| s.absolute_path.clone())
+                    .unwrap_or_default(),
+                &b.canonical
+                    .as_ref()
+                    .map(|s| s.absolute_path.clone())
+                    .unwrap_or_default(),
+            ),
 
             NodeSorter::ByCanonicalExtension => a
                 .canonical
@@ -1289,30 +1287,21 @@ pub enum NodeFilter {
 }
 
 impl NodeFilter {
-    fn apply(&self, node: &Node, input: &str) -> bool {
+    fn apply(&self, node: &Node, input: &str, regex: Option<&Regex>) -> bool {
         match self {
             Self::RelativePathIs => node.relative_path.eq(input),
-            Self::IRelativePathIs => {
-                node.relative_path.eq_ignore_ascii_case(input)
-            }
+            Self::IRelativePathIs => node.relative_path.eq_ignore_ascii_case(input),
 
             Self::RelativePathIsNot => !node.relative_path.eq(input),
-            Self::IRelativePathIsNot => {
-                !node.relative_path.eq_ignore_ascii_case(input)
-            }
+            Self::IRelativePathIsNot => !node.relative_path.eq_ignore_ascii_case(input),
 
-            Self::RelativePathDoesStartWith => {
-                node.relative_path.starts_with(input)
-            }
+            Self::RelativePathDoesStartWith => node.relative_path.starts_with(input),
             Self::IRelativePathDoesStartWith => node
                 .relative_path
                 .to_lowercase()
                 .starts_with(&input.to_lowercase()),
 
-            Self::RelativePathDoesNotStartWith => {
-                !node.relative_path.starts_with(input)
-            }
-
+            Self::RelativePathDoesNotStartWith => !node.relative_path.starts_with(input),
             Self::IRelativePathDoesNotStartWith => !node
                 .relative_path
                 .to_lowercase()
@@ -1324,69 +1313,51 @@ impl NodeFilter {
                 .to_lowercase()
                 .contains(&input.to_lowercase()),
 
-            Self::RelativePathDoesNotContain => {
-                !node.relative_path.contains(input)
-            }
+            Self::RelativePathDoesNotContain => !node.relative_path.contains(input),
             Self::IRelativePathDoesNotContain => !node
                 .relative_path
                 .to_lowercase()
                 .contains(&input.to_lowercase()),
 
-            Self::RelativePathDoesEndWith => {
-                node.relative_path.ends_with(input)
-            }
+            Self::RelativePathDoesEndWith => node.relative_path.ends_with(input),
             Self::IRelativePathDoesEndWith => node
                 .relative_path
                 .to_lowercase()
                 .ends_with(&input.to_lowercase()),
 
-            Self::RelativePathDoesNotEndWith => {
-                !node.relative_path.ends_with(input)
-            }
+            Self::RelativePathDoesNotEndWith => !node.relative_path.ends_with(input),
             Self::IRelativePathDoesNotEndWith => !node
                 .relative_path
                 .to_lowercase()
                 .ends_with(&input.to_lowercase()),
 
-            Self::RelativePathDoesMatchRegex => Regex::new(input)
+            Self::RelativePathDoesMatchRegex => regex
                 .map(|r| r.is_match(&node.relative_path))
                 .unwrap_or(false),
-            Self::IRelativePathDoesMatchRegex => {
-                Regex::new(&input.to_lowercase())
-                    .map(|r| r.is_match(&node.relative_path.to_lowercase()))
-                    .unwrap_or(false)
-            }
+            Self::IRelativePathDoesMatchRegex => regex
+                .map(|r| r.is_match(&node.relative_path.to_lowercase()))
+                .unwrap_or(false),
 
-            Self::RelativePathDoesNotMatchRegex => !Regex::new(input)
+            Self::RelativePathDoesNotMatchRegex => !regex
                 .map(|r| r.is_match(&node.relative_path))
                 .unwrap_or(false),
-            Self::IRelativePathDoesNotMatchRegex => {
-                !Regex::new(&input.to_lowercase())
-                    .map(|r| r.is_match(&node.relative_path.to_lowercase()))
-                    .unwrap_or(false)
-            }
+            Self::IRelativePathDoesNotMatchRegex => !regex
+                .map(|r| r.is_match(&node.relative_path.to_lowercase()))
+                .unwrap_or(false),
 
             Self::AbsolutePathIs => node.absolute_path.eq(input),
-            Self::IAbsolutePathIs => {
-                node.absolute_path.eq_ignore_ascii_case(input)
-            }
+            Self::IAbsolutePathIs => node.absolute_path.eq_ignore_ascii_case(input),
 
             Self::AbsolutePathIsNot => !node.absolute_path.eq(input),
-            Self::IAbsolutePathIsNot => {
-                !node.absolute_path.eq_ignore_ascii_case(input)
-            }
+            Self::IAbsolutePathIsNot => !node.absolute_path.eq_ignore_ascii_case(input),
 
-            Self::AbsolutePathDoesStartWith => {
-                node.absolute_path.starts_with(input)
-            }
+            Self::AbsolutePathDoesStartWith => node.absolute_path.starts_with(input),
             Self::IAbsolutePathDoesStartWith => node
                 .absolute_path
                 .to_lowercase()
                 .starts_with(&input.to_lowercase()),
 
-            Self::AbsolutePathDoesNotStartWith => {
-                !node.absolute_path.starts_with(input)
-            }
+            Self::AbsolutePathDoesNotStartWith => !node.absolute_path.starts_with(input),
             Self::IAbsolutePathDoesNotStartWith => !node
                 .absolute_path
                 .to_lowercase()
@@ -1398,65 +1369,104 @@ impl NodeFilter {
                 .to_lowercase()
                 .contains(&input.to_lowercase()),
 
-            Self::AbsolutePathDoesNotContain => {
-                !node.absolute_path.contains(input)
-            }
+            Self::AbsolutePathDoesNotContain => !node.absolute_path.contains(input),
             Self::IAbsolutePathDoesNotContain => !node
                 .absolute_path
                 .to_lowercase()
                 .contains(&input.to_lowercase()),
 
-            Self::AbsolutePathDoesEndWith => {
-                node.absolute_path.ends_with(input)
-            }
+            Self::AbsolutePathDoesEndWith => node.absolute_path.ends_with(input),
             Self::IAbsolutePathDoesEndWith => node
                 .absolute_path
                 .to_lowercase()
                 .ends_with(&input.to_lowercase()),
 
-            Self::AbsolutePathDoesNotEndWith => {
-                !node.absolute_path.ends_with(input)
-            }
+            Self::AbsolutePathDoesNotEndWith => !node.absolute_path.ends_with(input),
             Self::IAbsolutePathDoesNotEndWith => !node
                 .absolute_path
                 .to_lowercase()
                 .ends_with(&input.to_lowercase()),
 
-            Self::AbsolutePathDoesMatchRegex => Regex::new(input)
+            Self::AbsolutePathDoesMatchRegex => regex
                 .map(|r| r.is_match(&node.absolute_path))
                 .unwrap_or(false),
-            Self::IAbsolutePathDoesMatchRegex => {
-                Regex::new(&input.to_lowercase())
-                    .map(|r| r.is_match(&node.absolute_path.to_lowercase()))
-                    .unwrap_or(false)
-            }
+            Self::IAbsolutePathDoesMatchRegex => regex
+                .map(|r| r.is_match(&node.absolute_path.to_lowercase()))
+                .unwrap_or(false),
 
-            Self::AbsolutePathDoesNotMatchRegex => !Regex::new(input)
+            Self::AbsolutePathDoesNotMatchRegex => !regex
                 .map(|r| r.is_match(&node.absolute_path))
                 .unwrap_or(false),
-            Self::IAbsolutePathDoesNotMatchRegex => {
-                !Regex::new(&input.to_lowercase())
-                    .map(|r| r.is_match(&node.absolute_path.to_lowercase()))
-                    .unwrap_or(false)
-            }
+            Self::IAbsolutePathDoesNotMatchRegex => !regex
+                .map(|r| r.is_match(&node.absolute_path.to_lowercase()))
+                .unwrap_or(false),
         }
     }
 }
 
+// Regex doesn't implement Hash and Eq. But we need them.
+
+#[derive(Debug, Clone)]
+pub struct CmpRegex(Regex);
+
+impl std::hash::Hash for CmpRegex {
+    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+        self.0.as_str().hash(state)
+    }
+}
+
+impl PartialEq for CmpRegex {
+    fn eq(&self, other: &Self) -> bool {
+        self.0.as_str() == other.0.as_str()
+    }
+}
+
+impl Eq for CmpRegex {}
+
 #[derive(Debug, Clone, Eq, Hash, PartialEq, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct NodeFilterApplicable {
     pub filter: NodeFilter,
     pub input: String,
+
+    #[serde(skip)]
+    pub regex: Option<CmpRegex>,
 }
 
 impl NodeFilterApplicable {
     pub fn new(filter: NodeFilter, input: String) -> Self {
-        Self { filter, input }
+        use NodeFilter::*;
+
+        let regex = if matches!(
+            filter,
+            RelativePathDoesMatchRegex
+                | RelativePathDoesNotMatchRegex
+                | AbsolutePathDoesMatchRegex
+                | AbsolutePathDoesNotMatchRegex
+        ) {
+            Regex::new(&input).ok().map(CmpRegex)
+        } else if matches!(
+            filter,
+            IRelativePathDoesMatchRegex
+                | IRelativePathDoesNotMatchRegex
+                | IAbsolutePathDoesMatchRegex
+                | IAbsolutePathDoesNotMatchRegex
+        ) {
+            Regex::new(&input.to_lowercase()).ok().map(CmpRegex)
+        } else {
+            None
+        };
+
+        Self {
+            filter,
+            input,
+            regex,
+        }
     }
 
     fn apply(&self, node: &Node) -> bool {
-        self.filter.apply(node, &self.input)
+        self.filter
+            .apply(node, &self.input, self.regex.as_ref().map(|r| &r.0))
     }
 }
 
